#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

/*

gcc matrix_multiply_merge.c -O3 -o matrix_multiply_merge -Wall  -g
infiles=""; for((i=0;i<5000;i+=500)); do infiles="$infiles data/c_slice_$i.raw"; done
./matrix_multiply_merge $infiles data/c_5k_matrix.raw
 python verify_matrix.py data/a_5k_matrix.raw data/b_5k_matrix.raw data/c_5k_matrix.raw 

*/

double * matrix_load(const char * fname, int *n_rows, int *n_cols) {
  FILE *f = fopen(fname, "r");
  if(!f) {
    fprintf(stderr, "cannot open %s", fname); perror("");
    exit(1);
  }
  int size[2];
  int ret = fread(size, sizeof(int), 2, f);
  assert(ret == 2);
  *n_rows = size[0]; *n_cols = size[1];
  double *data = malloc(sizeof(double) * size[0] * size[1]); 
  ret = fread(data, sizeof(double), size[0] * size[1], f);
  assert(ret == size[0] * size[1]); 
  fclose(f);
  return data;
}

void matrix_store(const char * fname, int n_rows, int n_cols, const double *data) {
  FILE *f = fopen(fname, "w");
  if(!f) {
    fprintf(stderr, "cannot open %s", fname); perror("");
    exit(1);
  }
  int size[2] = {n_rows, n_cols};
  int ret = fwrite(size, sizeof(int), 2, f);
  assert(ret == 2);
  ret = fwrite(data, sizeof(double), size[0] * size[1], f);
  assert(ret == size[0] * size[1]); 
  fclose(f);
}






int main(int argc, char ** argv) {
  int n = -1;
  int m = 0;

  int n_input = argc - 2;
  double ** tables = malloc(sizeof(double*) * n_input); 
  int *mis = malloc(sizeof(int) * n_input);
  
  int i;
  for(i = 0; i < n_input; i++) {
    int mi, ni;
    tables[i] = matrix_load(argv[i + 1], &mi, &ni); 
    if(n < 0) n = ni; else assert(n == ni); 
    m += mi;
    mis[i] = mi;
  }

  double *c = malloc(sizeof(double) * m * n); 

  m = 0;
  for(i = 0; i < n_input; i++) {
    memcpy(c + m * n, tables[i], sizeof(double) * n * mis[i]);
    m += mis[i];
    free(tables[i]);
  }
  
  matrix_store(argv[argc - 1], m, n, c);

  free(c);
  
  return 0;
}

