
% argv is octave-specific
% in matlab, define a function and call it with -r "my_function(xx); quit"
args = argv()

% submit with
% for((i=1;i<30;i+=3)); do oarsub -nSL.$i "octave find_all_circles_job.m $i $[i+2]"; done

first_image = str2num(args{1,1})
last_image = str2num(args{2,1})


if exist ('OCTAVE_VERSION', 'builtin')
  % octave-specific function to access the image package functions
  pkg load image  
  
end

fprintf('processing images %d to %d\n', first_image, last_image);

path('matlab_functions', path)
path('detection_scale', path)


%%
%% declare global variables
%%
% build_scale_space
global r_init r_final r_iters r_factor r_final_factor sigma_smear max_filter_width max_sigma_r
global sigma_edge_init sigma_edge_final sigma_edge_factor % multiscale edge detection
% find_maxima
global nonmax_wid max_points unweighted_thresh_rel unweighted_thresh_abs border_width max_aspect_ratio
% find_sift
global sift_enlarge_factor

% build_scale_space
r_init = 3;
sigma_edge_init = 1;
sigma_edge_final = 3; 
sigma_edge_factor = 1.5; 
r_factor = sqrt(2); 
r_final_factor = 1/4; % positive if sigma_final is to be determined dynamically based on image size
r_final = 0; % if r_final_factor > 0 this is computed dynamically. Otherwise, it needs to be assigned manually here

sigma_smear = 0.075; % "fuzziness" of circle border
max_filter_width = 1000;
max_sigma_r = 5;

% find_maxima
nonmax_wid = 5;
max_points = 100;
unweighted_thresh_rel = 0.1; % number of votes in scale_space2 divided by circumference
unweighted_thresh_abs = 3; % absolute number of votes in scale_space2
border_width = 5;
max_aspect_ratio = 5;

sift_enlarge_factor = 1.5;

all_circles = []

% small case: process 3 images
% large case: process 30 images
for i=first_image:last_image
  image_name = sprintf('images/%d.png', i + 1000);

  fprintf('========== Processing image %s\n', image_name);
  
  I = load_image(image_name);
  [hgt wid] = size(I);
		
  %% extract edge segments at multiple scales
  fprintf('Extracting edge fragments...\n');
  edge_list = cell(sigma_edge_final,1);
  edge_arr = cell(sigma_edge_final,1);
  edge_sizes = cell(sigma_edge_final,1);
  
  for sigma_edge = sigma_edge_init:sigma_edge_final
    fprintf('sigma = %d/%d\n', sigma_edge, sigma_edge_final);
    [edge_list{sigma_edge}, edge_arr{sigma_edge}, edge_sizes{sigma_edge}] = find_edge_fragments(I, sigma_edge);
  end
  
  fprintf('Building scale spaces...\n');
  [scale_space, scale_space2] = build_scale_spaces(edge_arr, hgt, wid);
  
  fprintf('Finding maxima in scale spaces...\n');
  circles = find_maxima_double(I, scale_space, scale_space2);
  fprintf('%d circles found\n', size(circles,1));

  % [x y radius weight support sigma_edge]
  file_numbers  = i * ones(size(circles, 1), 1);
  all_circles = [all_circles; [file_numbers circles]];
end

save(sprintf('images/circles_%d:%d.mat', first_image, last_image),...
     'all_circles');

